import * as React from "react";
import { Form } from "antd";
import { isFn, isPromise, isObj, isArr } from "../utils/tool";
import renderOptions from "../lib/renderOptions";
import compMap from "../SCompMap/compMap";
import { compDidMount } from "../hooks/lifeCycle";

const { useState, useEffect } = React;
const defaultItemWidth = {
  RangePicker: 400
};
export default function SForm({
  initialData,
  filters,
  formProps,
  onChange: formChange,
  onFormDataChange,
  updateFormData,
  asFormChildren,
  getInnerApi
}: any) {
  const [formData, setFormData] = useState({});
  useEffect(() => {
    setFormData({ ...initialData }); // 动态依赖initialData
  }, [initialData]);
  const [itemChildren, setItemChildren] = useState({});
  const filterItemChange = (v: any, key: string, onChange: Function) => {
    if (v && v.target) v = v.target.value;
    const res = {
      ...formData,
      [key]: v
    };
    setFormData(res);
    if (isFn(onChange)) onChange(v);
    if (isFn(formChange)) formChange({ ...res });
  };

  useEffect(() => {
    isFn(updateFormData) && updateFormData(formData);
    isFn(onFormDataChange) && onFormDataChange({ ...formData });
  }, [formData]);
  const formItems = filters.map(
    ({
      Comp,
      type = "Input",
      label,
      key,
      formItemProps,
      render,
      onChange,
      options,
      style = {},
      props,
      required
    }: any) => {
      style.width = defaultItemWidth[type] || 300;
      const Item = compMap[type];
      compDidMount(() => {
        getInnerApi({ setFormData });
        const Children = render && render();
        if (options) return;
        if (isPromise(Children)) {
          Children.then((Child: any) => {
            if (
              isObj(Child) ||
              (isArr(Child) && Child[0].label && Child[0].value)
            )
              Child = renderOptions(Child);
            setItemChildren({
              ...itemChildren,
              [key]: Child
            });
          }).catch((err: any) => {
            console.log(`get ${key} options error :` + err);
          });
        } else {
          setItemChildren({
            ...itemChildren,
            [key]: Children
          });
        }
      });
      return Item ? (
        <Form.Item
          required={required}
          key={label}
          label={label}
          {...formItemProps}
        >
          {Comp ? (
            isFn(Comp) ? (
              <Comp />
            ) : (
              Comp
            )
          ) : (
            <Item
              allowClear={!required}
              value={formData[key]}
              onChange={(e: any) => {
                filterItemChange(e, key, onChange);
              }}
              style={style}
              {...props}
            >
              {options ? renderOptions(options) : itemChildren[key]}
            </Item>
          )}
        </Form.Item>
      ) : null;
    }
  );
  return asFormChildren ? (
    formItems
  ) : (
    <Form layout="inline" {...formProps}>
      {formItems}
    </Form>
  );
}
SForm.defaultProps = {
  filters: [],
  initialData: {}
};
