import * as React from "react";
import SFilter from "./SFilter";
import STable from "./STable";
import SModal from "./SModal";
import { getList as _getList } from "../lib/api";
import { dealParams, dealRangePickerParams } from "../utils/params";
import { isObj, isFn } from "../utils/tool";
import verifyParamsFn from "../lib/verifyParams";
import useColumns from "../hooks/useColumns";
import { compDidMount } from "../hooks/lifeCycle";
const { useState, useEffect, useRef } = React;

export default function SPage(props: any) {
  const {
    url,
    filters,
    tableColumns,
    beforeSearch,
    searchHandler,
    autoHandleRangePickerParams = true,
    initialData,
    initialSearch = true,
    tableProps,
    paginationProps,
    rowKey,
    innerApi,
    onTableDataChange,
    onResponseDataChange,
    onPagerChange,
    verifyParams = true,
    getList
  } = props;
  tableProps["rowKey"] = rowKey;
  const [formData, setFormData] = useState(initialData);
  const [tableData, setTableData] = useState(props.tableData);
  const [responseData, setResponseData] = useState({});
  const [pageSize, setPageSize] = useState(10);
  const [pageNum, setPageNum] = useState(1);
  const [totalNum, setTotalNum] = useState(0);
  const [modalChildren, setModalChildren] = useState(null);
  const [modalProps, setModalProps] = useState({});
  const [modalVisible, setModalVisible] = useState(false);
  const [searchLoading, setSearchLoading] = useState(false);
  const innerApiRef = useRef({} as any);
  onTableDataChange &&
    useEffect(() => {
      onTableDataChange(tableData, totalNum);
    }, [tableData, totalNum]);
  onResponseDataChange &&
    useEffect(() => {
      onResponseDataChange(responseData);
    }, [responseData]);
  onPagerChange &&
    useEffect(() => {
      onPagerChange(pageNum, pageSize);
    }, [pageNum, pageSize]);
  const requiredParams: string[] = [];
  filters.forEach(({ required, key }: any) => {
    if (required) requiredParams.push(key);
  });
  const getBeforeSearchParams = () => {
    let opt = { ...formData };
    if (isFn(beforeSearch)) {
      const handlerRes = beforeSearch({ ...opt });
      if (handlerRes === false) return false;
      isObj(handlerRes) && (opt = handlerRes);
    }
    return opt;
  };
  const getTableList = async (opt: any) => {
    try {
      setSearchLoading(true);
      const resData: any = await getList(url, opt);
      const { data, data: { list, total_num } = {} as any } = resData;
      setTotalNum(total_num);
      setTableData(list || []);
      setResponseData(resData || {});
      return data;
    } finally {
      setSearchLoading(false);
    }
  };
  const getListWithDealParams = async (
    params?: any,
    fullCoverageParams?: object
  ) => {
    let opt: any = null;
    if (fullCoverageParams) {
      opt = fullCoverageParams;
    } else {
      opt = getBeforeSearchParams();
      if (opt === false) return;
      Object.assign(
        opt,
        { page_num: pageNum, page_size: pageSize },
        isObj(params) ? params : null
      );
      if (autoHandleRangePickerParams) opt = dealRangePickerParams(opt);
      if (tableProps.pagination === false) {
        opt.page_size = opt.page_num = undefined;
      }
      opt = dealParams(opt);
    }
    return await getTableList(opt);
  };
  const _searchHandler = () => {
    searchHandler && searchHandler();
    if (
      verifyParams &&
      !verifyParamsFn(dealParams({ ...formData }), requiredParams)
    )
      return;
    setPageNum(1);
    return getListWithDealParams({ page_num: 1 });
  };
  const openModal = (
    record: any,
    modalRender: Function,
    modalProps: any = {}
  ) => {
    setModalVisible(true);
    setModalChildren(modalRender && modalRender(record));
    setModalProps(modalProps);
  };
  const closeModal = () => {
    setModalVisible(false);
  };

  const pageChange = (pageNum: any, pageSize: any) => {
    setPageNum(pageNum);
    pageSize && setPageSize(pageSize);
    getListWithDealParams({
      page_num: pageNum,
      page_size: pageSize
    });
  };
  const columns = useColumns(filters, tableColumns, openModal);
  if (initialSearch)
    compDidMount(() => {
      _searchHandler();
    });
  compDidMount(() => {
    isFn(innerApi) && innerApi(innerApiRef.current);
  });
  useEffect(() => {
    Object.assign(innerApiRef.current, {
      fetchTableList: getListWithDealParams
    });
  });
  return (
    <div>
      <SFilter
        updateFormData={setFormData}
        searchLoading={searchLoading}
        {...props}
        searchHandler={_searchHandler}
      />
      {props.children}
      <STable
        tableData={tableData}
        tableColumns={columns}
        pageChange={pageChange}
        pageSize={pageSize}
        pageNum={pageNum}
        totalNum={totalNum}
        loading={searchLoading}
        tableProps={tableProps}
        paginationProps={paginationProps}
      />
      <SModal closeModal={closeModal} props={modalProps} visible={modalVisible}>
        {modalChildren}
      </SModal>
    </div>
  );
}
SPage.defaultProps = {
  filters: [],
  tableColumns: [],
  tableProps: {},
  initialData: {},
  getList: _getList
};
export const getSPage = (getList: any) => {
  return function SPageWithGetList(props: any) {
    return <SPage {...props} getList={getList}></SPage>;
  };
};
